#!/usr/bin/perl

# I have written a little perl script that runs the test suite in
# parallel on suitable machines. The syntax is:
#
# run_parallel.pl cloudy.exe 16
#
# where the first parameter is the name of the Cloudy executable. It
# should either be in your search path, or the full pathname should
# be used. The second number is the number of processors to use. It
# can be any number >= 1. However, using numbers larger than roughly
# 16 will not give any speed advantage since the time will then be
# set by the longest running test sims.
#
# The script uses "gmake" (GNU make) to run the test cases. In case
# that doesn't exist, you can try replacing the two calls below with
# "make". If the version is not too old, that should work as well
# provided it supports the -j parameter for parallel execution.
# The script may need some other tinkering too, but I find it very
# useful here.
# Peter van Hoof

if( "$ARGV[0]" eq "" ) {
    print "The name for the cloudy binary must be supplied...\n";
    exit;
}

if( "$ARGV[1]" eq "" ) {
    print "The number of processors must be supplied...\n";
    exit;
}

# clean up first
if( -e "./clean_tsuite.pl" ) {
    system "./clean_tsuite.pl";
}
else {
    die "./clean_tsuite.pl not found!";
}

$all = "all: ";
$rules = "";
$i = 0;

# this funny sequence is to assure that the heavy jobs are upfront...
while( defined( $input = glob("h*.in o*.in d*.in b*.in l*.in n*.in p*.in [A-Zace-gi-kmq-z]*.in") ) ) {
    $i++;
    $output = $input;
    $output =~ s/\.in/.out/gi;
    $all = "$all$output ";
    if( $i%4 == 0 ) {
	$all = "$all\\\n\t";
    }
    $rules = "$rules$output: $input\n";
    $rules = "$rules\t$ARGV[0] < $input > $output\n"
}
open( foo, ">Makefile" );
print foo $all;
print foo "\n";
print foo $rules;
close foo;

# this does the heavy lifting...
# try make instead if gmake doesn't exist on your system
system "gmake -i -j $ARGV[1]";

# this was produced by the optimizers and is not really part of the test suite
# its presence will fool the checkall.pl script....
unlink "optimal.in";

if( $ARGV[1] > 1 ) {
# just in case func_trans_read was run before func_trans_punch was complete
   unlink "func_trans_read.out";
# try make instead if gmake doesn't exist on your system
   system "gmake -i -j $ARGV[1]";
}

# now do the checking
system "./checkall.pl";
